/*   compare_user.c: 
     Supporting functions for compare. These function control the data 
     selection, and can/should be modified by the user to display or 
     color some subset of the data. These functions are called in the 
     read_<file-type>() programs and have the following selection:
                     if (Condition) include particle
		     (*particle).key=GetKey()

int GetKey(struct particle *p) 
int Condition(rank,X,Y,Z,VX,VY,VZ,KEY,PHI)
 */

#include "compare.h"
#include <math.h>
/*==============  BEGIN GetKey(p,nbodies) =================================
 Assigns a key to the particle based on its parameters. At present,
 key is assigned based on a particles rank w.r.t the total number of
 particles. I assume 2 galaxies, so the first half have keys < 1000, 
 and the 2nd have keys > 1000. Every 2048 particles are given a different
 key. This is useful for reading in the t9d_22k files from Zeno.           */

int GetKey(struct particle *p,int ntotal) 
{
	int key_index;
	
	if( (*p).rank<(ntotal>>1) ) key_index = 1+ ( (*p).rank>>11 );
	else key_index = 1001 + ( ( (*p).rank - (ntotal>>1) )>>11);
	if( debug && ((*p).rank%1000==0))
	  printf("np=%d, rank=%d, key=%d\n",ntotal,(*p).rank,key_index);
	return (key_index);
}
/*==============  END GetKey(p,nbodies) ===================================*/

/*============== BEGIN Condition(RANK,X,Y,Z,VX,VY,VZ,KEY,PHI)==============*/

int Condition(int rank, float X, float Y, float Z,
	      float VX, float VY, float VZ, int KEY, float PHI)
{
  int cond1,cond2,cond3,cond4,ndisk;
  
  if(debug) 
    printf("Condition: %5d %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %5d %9.2f\n",
	   rank,X,Y,Z,VX,VY,VZ,KEY,PHI);
  
  ndisk=KEY>>1;
  cond1=( inbtw(rank,4096,ndisk) && (rank%2==0) );
  cond2=( inbtw(rank,ndisk+4096,KEY) && (rank%2==0) );
  cond3=( (rank<4096) && (rank%5==0) );
  cond4=( inbtw(rank,ndisk,ndisk+4096) && (rank%5==0) );
  return( cond1 || cond2 || cond3 || cond4); 
}
/*==================END Condition( X,Y,Z,VX,VY,VZ,KEY,PHI)=====================*/
