#define DATACOLOR 0
#include "compare.h"

/* 	Reads in particle data file. See pf.hdr for the format
 *	Also loads or calculates the following global variables:
 *		int: 	npoints[type]; 
 *		float:	V_sys; scale_r; scale_v; in2out_r; in2out_v; min_flux;
 *
 */

struct particle *read_data(char filename[50])
{
	FILE  *in_file; 
	char linebuff[132];
	struct particle *pp,*p_data;
	int end_of_header=0;
	int count=0;
	int rank=0;
	float x_min=1000;
	float x_max=-1000;
	float y_min=1000;
	float y_max=-1000;
	float V_max,V_min,deltaV;
	float x,y,z,vx,vy,vz;
	float flux,max_flux,rad;
	int i,numin,linenum;
	unsigned long key; 
	debug=NO;
	p_data = NULL; 

	printf("Trying to open data file %s\n", filename);
	if((in_file=fopen(filename,"r")) ==NULL) 
	{
	  fprintf(stderr,"Couldnt open %s for reading. exiting...\n",filename);
	  npoints[type]=0;
	  exit(-1);
	  return(p_data);
	}

/******	FIRST READ IN HEADER. PARTICLE DATA STARTS AFTER THE $$$ SEQUENCE *********/

       V_sys=0;scale_r[type]=1;scale_v[type]=1;min_flux=0;V_min=200000;V_max=0;

	end_of_header=NO;
	linenum=0;
	while ( (fgets(linebuff,132,in_file)!=NULL) && (!end_of_header) )
	{
		linenum++;
		if (strncmp(linebuff,"$$$",3) == 0 ) end_of_header=YES; 
		else if(linenum>50) 
		{
                printf("Read in too many lines. Check to make sure that cc.hdr has been added to your data file\n");
		end_of_header=YES;
		exit(0);
		}
		else if (strncmp(linebuff,"npoints",7)   == 0 ) sscanf(linebuff,"npoints = %d", &npoints[type]);
		else if (strncmp(linebuff,"V_sys",5)   == 0 ) sscanf(linebuff,"V_sys = %f", &V_sys);
		else if (strncmp(linebuff,"scale_r",7) == 0 ) sscanf(linebuff,"scale_r = %f", &scale_r[type]);
		else if (strncmp(linebuff,"scale_v",7) == 0 ) sscanf(linebuff,"scale_v = %f", &scale_v[type]);
		else if (strncmp(linebuff,"min_flux",7) == 0 ) sscanf(linebuff,"min_flux = %f", &min_flux);
 	}
	printf(" npoints= %d  V_sys=%f \n scale_r= %f \n scale_v= %f \n min_flux=%f \n ",
				npoints[type],V_sys,scale_r[type],scale_v[type],min_flux);

	if(linenum<7) printf("Didnt read in enough lines. Check to make sure that pf.hdr is in your particle file\n");

/****** FREE UP ANY EXISTING DATA, AND MALLOC THE APPROPRIATE AMOUNT OF MEMORY***************/

	if(points[type] != NULL) free(points[type]);
	if(npoints[type] != 0) 
	{
	    p_data = (struct particle *) malloc(npoints[type]*sizeof(*p_data));
	    if(p_data == NULL) 
	    {
		printf("<sorry, no memory for DATA data>\n") ;
		return ;
	    }
	}

	printf("Mallocated %d bytes of data. \n",npoints[type]*sizeof(*p_data));

/****** NOW READ THROUGH THE REST OF THE FILE, ONLY ENTERING THE DATA IF A CONDITION IS SATISFIED *********/

	pp=p_data;
	while(fgets(linebuff,132,in_file)!=NULL)
	{
 linenum++;
 numin=sscanf(linebuff,"%d %f %f %f %f %f %f %d %f",&rank,&x,&y,&z,&vx,&vy,&vz,&key,&flux);
		if (numin==9)
		{
		if((type==DATA)||((type==MODEL)&&(count<4024)&&(rank>1024)))
			{
			(*pp).rank=rank;
			(*pp).x= x*scale_r[type];
			(*pp).y= y*scale_r[type];
			(*pp).z= z*scale_r[type];
			(*pp).rad=10*scale_r[type];
			(*pp).vx=vx*scale_v[type];
			(*pp).vy=vy*scale_v[type];
			(*pp).vz=vz*scale_v[type];
			(*pp).key=key;
			(*pp).phi=flux;

/* I assume the desired key will be in the input file. 
 * If not you will need to set (*pp).key=GetKey(pp) here. 
 * ACTUALLY, ONLY USES KEY FOR DRAWING IF DRAWCIRCS IS CHOSEN
 * IN REFRESH; KEY IS NOW USED TO IDENTIFY SPECIFIC PARTICLES
 * GetKey is used just to set up other dynamical quantities.
 */
   if(type==MODEL) {i=GetKey(pp,numin);(*pp).rad=15;}
   else{
   /*** THIS MAKES SURE POINTS ARE BETWEEN 5 AND 30 PIXELS ***/
       (*pp).rad=max(5,min(10*flux/min_flux,30));
       }
			count++;
			pp++;
			if(V_min>vz) V_min=vz;
			else if(V_max<vz) V_max=vz;
			if(x>x_max) x_max=x;
			else if(x<x_min) x_min=x;
			if(y>y_max) y_max=y;
			else if(y<y_min) y_min=y;
			if(flux>max_flux) max_flux=flux;
			numin=0;
			}
		}
         else if(strncmp(linebuff,"$$$",3) != 0 ) {
            printf("NOT ENOUGH DATA ON LINE # %d. LINE SKIPPED. LINE FOLLOWS:\n",linenum);
            printf("Line %d: %s",linenum,linebuff);
          }
        }
        printf("\n Read in %d lines from data file \n",linenum);
	fclose(in_file);
        printf("File closed.\n\n");

        npoints[type]=count;

        /*** CALCULATE SCREEN SCALE FACTORS ***/
        if(type==DATA){
          deltaV=V_max-V_min+1;

          in2out_r=0.8*boxsz/scale_r[type]/max(x_max-x_min+1,y_max-y_min+1);
          if(in2out_r<=0)in2out_r=0.0001;
          in2out_v=boxsz/4/scale_v[type]/deltaV;
          if(in2out_v<=0)in2out_v=0.0001;
          if(debug){
          printf(" npoints= %d,  max_flux= %f\n", npoints[type],max_flux);
          printf(" x_min  = %f,  x_max   = %f\n y_min   = %f, y_max   = %f\n",
                 x_min,x_max,y_min,y_max);
          printf(" V_min  = %f,  V_max   = %f\n", V_min, V_max);
          printf(" boxsz  = %d,  in2out_r= %f, in2out_v= %f\n", 
                 boxsz, in2out_r, in2out_v);
          }
        }
        return(p_data);

}





