/*===================BEGIN read_SNAP(filename)=========================
  Reads simulation files output from TIPSY. These have 
  mass[i], pos[1][i], pos[2][i], ... vel[3][i], i=1...nbodies.
  I only read in parameters used by compare. To read in ALL Tipsy 
  parameters, see read_SNAP_original().
  A condition is run on the file, and a "key" is assigned according 
  to the function GetKey().  Returns two copies of the model data.   ***/

#include "compare.h"

struct particle *read_snap(char filename[50])
{
    FILE  *infile; 
    int number,count; int i;
    char cmmd[50];
    float junk,X,Y,Z,VX,VY,VZ;
    float PHI, KEY;
    int ndim,nbodies,ndark,nstar,ngas;
    double time;
    struct particle *p,*pp,*p_model,*ptemp;
    debug=NO;
    type=MODEL;
    
    p_model=NULL;

for(i=0;i<50;i++)
  {
   if(filename[i]=='\0')
    {
      if(filename[i-2]=='.'&&filename[i-1]=='Z')
       {
       printf("Trying to open and zcat model file %s\n", filename);

       sprintf(cmmd,"zcat %s > temp",filename);
        if(system(cmmd) > 0 ) exit(0);
       sprintf(filename,"%s","temp"); 
        if((infile=fopen(filename,"r")) ==NULL) 
         {
	 fprintf(stderr,"Couldnt open %s for reading. exiting...\n",filename);
	 npoints[MODEL]=0;
	 exit(-1);
	 return(p_model); 
         }
       sprintf(cmmd," rm %s","temp");
        if(system(cmmd) > 0 ) exit(0);
       break;
       }
	
     else if(filename[i-3]=='.'&&filename[i-2]=='g'&&filename[i-1]=='z')
       {
       printf("Trying to open and gunzip model file %s\n", filename);

       sprintf(cmmd,"cp %s temp.gz",filename);
        if(system(cmmd) > 0 ) exit(0);
       sprintf(cmmd," gunzip %s","temp.gz");
        if(system(cmmd) > 0 ) exit(0);
       sprintf(filename,"%s","temp"); 
        if((infile=fopen(filename,"r")) ==NULL) 
        {
        fprintf(stderr,"Couldnt open %s for reading. exiting...\n",filename);
	npoints[MODEL]=0;
	exit(-1);
	return(p_model);
        }
       sprintf(cmmd," rm %s","temp");
        if(system(cmmd) > 0 ) exit(0);     
       break;
        }

      else if(filename[i-5]=='.'&&filename[i-4]=='d'&&filename[i-3]=='a'&&filename[i-2]=='t'&&filename[i-1]=='a')
        {
	  if((infile=fopen(filename,"r")) ==NULL) 
          {
	  fprintf(stderr,"Couldnt open %s for reading. exiting...\n",filename);
	  npoints[MODEL]=0;
	  exit(-1);
	  return(p_model);
          }
	break;
	}
       else 
	{
	printf("Are you sure that this is a SNAP file? Exiting...\n");
	exit(1);
	}
      }
  }

 
    if(debug) printf("Well, its open. On we go....\n");
    count=fscanf(infile, " %d %d %d ", &nbodies, &ngas, &nstar) ;
    if ( (count == EOF) || (count==0) || (nbodies==0) ){
      fprintf(stderr,"improper format or nbodies=0. See ya.\n");
      exit(1);
    }
    ndark = nbodies - nstar - ngas ;
    count=fscanf(infile,"%d",&ndim) ;
    if ( (count == EOF) || (count==0) || (ndim!=3) ){
      fprintf(stderr,"improper format or ndim!=3. See ya.\n");
      exit(1);
    }
    fscanf(infile,"%lf",&time) ;

    /*** FREE UP ANY EXISTING DATA, AND MALLOC THE APPROPRIATE AMOUNT 
	 OF MEMORY***/

    if(points[MODEL] != NULL) free(points[MODEL]);
    p_model = (struct particle *) malloc(nbodies*sizeof(*p_model));
    if(p_model == NULL) {
      printf("<sorry, no memory for model data>\n") ;
      return(p_model);
    }
    if(debug) printf("Mallocated %d bytes of model data. \n",
		     nbodies*sizeof(*p_model));

    /*** READ THE FILE, FIRST PASSING OVER THE MASSES, WHICH WE DO NOT USE ***/
    for(count=0;count<nbodies;count++){
      fscanf(infile,"%f%*[, \t\n]",&junk);
    }

    /*** ON THE FIRST PASS THROUGH, FILL IN OTHER point STRUCTURES. ASSIGN A 
	 KEY DEPENDING ON IF IT IS A DARK, STAR, OR GAS PARTICLE. ***/
    count=0;
    for(pp=p_model;pp<(p_model+nbodies);pp++){
      fscanf(infile,"%f%*[, \t\n]",&pp->x);
      if(count<ndark) (*pp).key=1;       
      if( (nstar!=0) && inbtw(count,ndark,ndark+nstar) ) (*pp).key=2;   
      if( (ngas!=0) && inbtw(count,ndark+nstar,nbodies) ) (*pp).key=3; 
      (*pp).rank=count++;
      (*pp).phi=-0.001;                   /* I could read this in if I wanted */
    }
    for(pp=p_model;pp<(p_model+nbodies);pp++){
      fscanf(infile,"%f%*[, \t\n]",&pp->y);
    }
    for(pp=p_model;pp<(p_model+nbodies);pp++){
      fscanf(infile,"%f%*[, \t\n]",&pp->z);
    }
    for(pp=p_model;pp<(p_model+nbodies);pp++){
      fscanf(infile,"%f%*[, \t\n]",&pp->vx);
    }
    for(pp=p_model;pp<(p_model+nbodies);pp++){
      fscanf(infile,"%f%*[, \t\n]",&pp->vy);
    }
    for(pp=p_model;pp<(p_model+nbodies);pp++){
      fscanf(infile,"%f%*[, \t\n]",&pp->vz);
    }
    fclose(infile);

    /*** OK, pp contains all the data. We will only keep some
	 subset of these, depending on CONDITION ***/

    scale_r[MODEL]=SCALE_R;
    scale_v[MODEL]=SCALE_V;
    number=0;
    count=0;
    p=p_model;
    for(pp=p_model;pp<(p_model+nbodies);pp++){
      X=  scale_r[MODEL]*(*pp).x;
      Y=  scale_r[MODEL]*(*pp).y;
      Z=  scale_r[MODEL]*(*pp).z;
      VX= scale_v[MODEL]*(*pp).vx;
      VY= scale_v[MODEL]*(*pp).vy;
      VZ= scale_v[MODEL]*(*pp).vz;
      PHI=scale_v[MODEL]*scale_v[MODEL]*(*pp).phi;
      KEY= (*pp).key;
      count++;
      if( Condition(count,X,Y,Z,VX,VY,VZ,KEY,PHI) ) 
      {
	number++;
	(*p).rank= count;
	(*p).x= X;
	(*p).y= Y;
	(*p).z= Z;
	(*p).vx= VX;
	(*p).vy= VY;
	(*p).vz= VZ;
	(*p).key=GetKey(pp); 
	(*p).phi= PHI;
	(*p).rad= 15;
	p++;
      }
    }
  printf("File successfully read. %d model points passed Condition()\n",number);
  npoints[type]=count;
  /*
    if(number<count){
      ptemp = (struct particle *) malloc(number*sizeof(*p_model));
      if(ptemp == NULL) {
	printf("<sorry, no memory for model data>\n") ;
	return(ptemp);
	if(debug) printf("Mallocated %d bytes of model data. \n",
		     number*sizeof(*p_model));
      }
      ptemp=p_model;
      free(p_model);
      p_model=ptemp;
      }*/
    return(p_model);
}










