/*===================BEGIN read_Zeno(filename)=========================
  Reads simulation files output from stoa in Zeno (Barnes 1999). 
  These are different from the Tipsy Snap files, in that they have
  mass[i], pos[*][i], vel[*][i], i=1...nbodies, instead of
  mass[i], pos[1][i], pos[2][i], ... vel[3][i], i=1...nbodies.
  Assumes input file is binary, and runs stoa on it (so Zeno/bin 
  better be in your path).
  Assigns a "key" interger according to the function GetZenoKey(). 
  Returns two copies of the model data.                             ***/

#include "compare.h"

struct particle *read_zeno(char filename[50])
{
    FILE  *infile; 
    int number,count;
    char cmmd[50];
    float junk,X,Y,Z;
    int ndim ;
    int nbodies ;
    double time;
    struct particle *p,*p_model;

    debug=NO;

    p_model=NULL;
    printf("Trying to open and stoa model file %s\n", filename);
    sprintf(cmmd,"stoa %s temp options=Position,Velocity",filename);
    if(system(cmmd) > 0 ) exit(0);
    sprintf(filename,"%s","temp"); 
    if((infile=fopen(filename,"r")) ==NULL) 
    {
      fprintf(stderr,"Couldnt open %s for reading. exiting...\n",filename);
      npoints[MODEL]=0;
      exit(-1);
      return(p_model);
    }
    type=MODEL;

    if(debug) printf("Well, its open. On we go....\n");
    count=fscanf(infile, " %d ", &nbodies) ;
    if ( (count == EOF) || (count==0) || (nbodies==0) ){
      fprintf(stderr,"improper format or nbodies=0. See ya.\n");
      exit(1);
    }
    count=fscanf(infile,"%d",&ndim) ;
    if ( (count == EOF) || (count==0) || (ndim!=3) ){
      fprintf(stderr,"improper format or ndim!=3. See ya.\n");
      exit(1);
    }
    fscanf(infile,"%lf",&time) ;

    /*** FREE UP ANY EXISTING DATA, AND MALLOC THE APPROPRIATE AMOUNT 
	 OF MEMORY***/

    if(points[MODEL] != NULL) free(points[MODEL]);
    p_model = (struct particle *) malloc(nbodies*sizeof(*p_model));
    if(p_model == NULL) {
      printf("<sorry, no memory for model data>\n") ;
      return(p_model);
    }
     printf("Mallocated %d bytes of model data. \n",
		     nbodies*sizeof(*p_model));

    /*** FIRST, READ THROUGH MASSES, SINCE THESE ARE NOT USED HERE 
    for(count=0;count<nbodies;count++){
      fscanf(infile,"%f%*[, \t\n]",&junk);
    }***/

    /*** NOW READ THROUGH X,Y,Z, BUT ONLY ENTER SOME SUBSET OF THE DATA ***/
    scale_r[MODEL]=SCALE_R;
    number=0;
    p=p_model;
    for(count=0;count<nbodies;count++){
      fscanf(infile,"%e %e %e",&X, &Y, &Z);
      if( (count>4095) && (count%10==0) ){
	  number++;
	  (*p).x= scale_r[MODEL]*X;
	  (*p).y= scale_r[MODEL]*Y;
	  (*p).z= scale_r[MODEL]*Z;
	  (*p).rank= count;
	  (*p).phi= -0.001;
	  /*** I AM ASSUMING FOR NOW THAT THE FIRST 4096 
	       PARTICLES ARE HALO, THEN THERE ARE NINE DISKS, 
	       EACH WITH 2048 PARTICLES, AND I WANT TO ASSIGN
	       THESE KEYS=1...9                              ***/
	  (*p).key=(count-4096+2049)/2048;    
	  (*p).rad= 15;
	  p++;
      }
    }

    /*** NOW READ THROUGH VX,VY,VZ, BUT ONLY ENTER SOME SUBSET OF THE DATA ***/
    scale_v[MODEL]=SCALE_V;
    number=0;
    p=p_model;
    for(count=0;count<nbodies;count++){
      fscanf(infile,"%e %e %e",&X, &Y, &Z);
      if( (count>4095) && (count%10==0) ){
	  number++;
	  (*p).vx= scale_v[MODEL]*X;
	  (*p).vy= scale_v[MODEL]*Y;
	  (*p).vz= scale_v[MODEL]*Z;
	  p++;
      }
    }

    npoints[MODEL]=number;
    printf("\nRead in %d model points..",number);
    
    fclose(infile);
    if(system("rm temp") > 0 ) exit(-3); 

    printf("File closed.\n\n");
    return(p_model);
}
/*===================END read_Zeno(filename)==============================*/










